/************************************************************************
*
* dis980 - Disassembles objects from asm980
*
* Changes:
*   08/02/11   DGP   Original. Hacked from dis990.
*   09/27/11   DGP   Added cpu models and Calder 980 support.
*	
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#include "disdef.h"

int pcreg; /* the pc */
int absolute = FALSE;
int symbolcount = 0;
int pgmlen = 0;
int entry = -1;
int modcount = 0;
int cardmode;
int model;
char inbuf[MAXLINE];
char idtbuf[IDTSIZE+2];

SymNode *symbols[MAXSYMBOLS];
Module modules[MAXMODULES];

int16 memory[MEMSIZE];
Memory memctl[MEMSIZE];

char *regnames[16] =
{
   "A", "E", "X", "M", "S", "L", "B", "P",
   "R0", "R1", "R2", "R3", "R4", "R5", "R6", "R7"
};

/***********************************************************************
* printsymbols - Print the symbol table.
***********************************************************************/

static void
printsymbols (void)
{
   int gotone;
   int i;

   gotone = FALSE;
   for (i = 0; i < symbolcount; i++)
   {
      if (symbols[i]->flags & GLOBAL)
      {
	 printf ("\tDEF\t%s\n", symbols[i]->symbol);
	 gotone = TRUE;
      }
      else if (symbols[i]->flags & EXTERNAL)
      {
	 printf ("\tREF\t%s\n", symbols[i]->symbol);
	 gotone = TRUE;
      }
   }
   if (gotone)
      printf ("*\n");
}

/***********************************************************************
* printabsgbl - Print absolute global symbols.
***********************************************************************/

static void
printabsgbl (void)
{
   SymNode *s;
   int gotone;
   int i;

   gotone = FALSE;
   for (i = 0; i < symbolcount; i++)
   {
      s = symbols[i];
      if ((s->flags & GLOBAL) && !(s->flags & RELOCATABLE))
      {
	 printf ("%s\tEQU\t>%04X\n", s->symbol, s->value);
	 gotone = TRUE;
      }
   }
   if (gotone)
      printf ("*\n");
}

/***********************************************************************
* Main procedure
***********************************************************************/

int
main (int argc, char **argv)
{
   FILE *infd = NULL;
   char *bp;
   char *infile;
   int i;
   int status = 0;
  
#ifdef DEBUG
   fprintf (stderr, "dis980: Entered:\n");
   fprintf (stderr, "args:\n");
   for (i = 1; i < argc; i++)
      fprintf (stderr, "   arg[%2d] = %s\n", i, argv[i]);
#endif

   /*
   ** Clear symboltable
   */

   for (i = 0; i < MAXSYMBOLS; i++)
      symbols[i] = NULL;

   /*
   ** Scan off the the args.
   */

   idtbuf[0] = '\0';
   infile = NULL;
   model = CPU_TI980B;

   for (i = 1; i < argc; i++)
   {
      bp = argv[i];

      if (*bp == '-')
      {
         bp++;
	 if (*bp == 'm')
	 {
	    i++;
	    if (i >= argc) goto USAGE;
	    if (!strcmp (argv[i], "TI980"))
	       model = CPU_TI980;
	    else if (!strcmp (argv[i], "TI980A"))
	       model = CPU_TI980A;
	    else if (!strcmp (argv[i], "TI980B"))
	       model = CPU_TI980B;
	    else if (!strcmp (argv[i], "CDD980"))
	       model = CPU_CDD980;
	    else
	    {
	       fprintf (stderr, "Unsupported CPU model: %s\n", argv[i]);
	       goto USAGE;
	    }
	 }
	 else
	 {
	 USAGE:
	    printf ("dis980 - version %s\n", VERSION);
	    printf ( "usage: dis980 [-m model] infile.bin...\n");
	    return (ABORT);
	 }
      }

      else
      {
	 if (infile != NULL) goto USAGE;
         infile = argv[i];
      }

   }

   if (!infile) goto USAGE;

   /*
   ** Open the input.
   */

   if ((infd = fopen (infile, "r")) == NULL)
   {
      sprintf (inbuf, "dis980: Can't open input file %s", infile);
      perror (inbuf);
      exit (1);
   }

   /*
   ** Clear memory.
   */

   memset (&memory, '\0', sizeof(memory));
   for (i = 0; i < MEMSIZE; i++)
   {
      memset (&memctl[i], '\0', sizeof (Memory));
   }

   /*
   ** Load the object.
   */

   cardmode = FALSE;
   pcreg = 0;
   status = disloader (infd, pcreg, infile);

   /*
   ** Output module preamble.
   */

   printf ("*\n");
   printf ("* Module %s \n", idtbuf);
   if (modules[0].time[0])
      printf ("* Created on %s %s by %s\n",
	    modules[0].time, modules[0].date, modules[0].creator);
   else
      printf ("* Created by %s\n",
	    modules[0].creator);
   printf ("*\n");

   if (idtbuf[0])
   {
      printf ("\tIDT\t%s\n", idtbuf);
      printf ("*\n");
   }

   for (i = 0; i < (model == CPU_CDD980 ? 16 : 8); i++)
      printf ("%s\tEQU\t%d\n", regnames[i], i & 0x7);
   printf ("*\n");

   printsymbols();
   printabsgbl();

   /*
   ** Process it.
   */

   pcreg = 0;
   disprog (FALSE);

   if (entry >= 0)
   {
      SymNode *s;

      if ((s = vallookup (entry, FALSE, FALSE)) != NULL)
	 printf ("\tEND\t%s\n", s->symbol);
      else
	 printf ("\tEND\t>%X\n", entry);
   }
   else
      printf ("\tEND\n");

   /*
   ** Close the input
   */

   fclose (infd);

   return (status == 0 ? NORMAL : ABORT);
}
