/***********************************************************************
*
* disdef.h - Disassembler header for the TI 980 computer.
*
* Changes:
*   08/02/11   DGP   Original.
*   09/27/11   DGP   Added cpu models and Calder 980 support.
*
***********************************************************************/

#include <stdio.h>
#include "utiltypes.h"

/*
** Definitions
*/

#define NORMAL 0
#define ABORT  12

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define VERSION "1.3.1"

#define EOFSYM ':'

#define MAXLINE 256
#define LINESPAGE 60
#define MAXFILES 30
#define IDTSIZE 6

/*
** CPU Model
*/

enum cpumodels
{
   CPU_TI980, CPU_TI980A, CPU_TI980B, CPU_CDD980
};

/*
** Instruction types
*/

#define MAX_INST_TYPES 11

enum optypes
{
   TYPE_1=0, TYPE_2,  TYPE_3,  TYPE_4,  TYPE_5,  TYPE_6,  TYPE_7,
   TYPE_8,   TYPE_9,  TYPE_10, TYPE_11, TYPE_ILL
};

/*
** Memory
*/

#define MEMSIZE 65768

typedef struct
{
   char tag;
   int	value;
   int	relocatable;
   int  external;
   int  global;
   int  used;
} Memory;

#define GETMEM(m) memory[m]
#define PUTMEM(m,v) memory[m] = v

/*
** Symbol table
*/

#define MAXSYMLEN 6
#define MAXSYMBOLS 5000

#define RELOCATABLE 0x000000001
#define EXTERNAL    0x000000002
#define GLOBAL      0x000000004
#define MULDEF      0x000000010
#define UNDEF       0x000000020

typedef struct
{
   char symbol[MAXSYMLEN+2];
   char module[MAXSYMLEN+2];
   int flags;
   int value;
   int order;
} SymNode;

/*
** Module table
*/

#define MAXMODULES 2

typedef struct 
{
   char name[IDTSIZE+2];
   char date[12];
   char time[12];
   char creator[12];
   char objfile[256];
   int origin;
   int length;
} Module;

/*
** Object tags
*/

#define IDT_TAG		'0'
#define ABSENTRY_TAG	'1'
#define RELENTRY_TAG	'2'
#define RELEXTRN_TAG	'3'
#define ABSEXTRN_TAG	'4'
#define RELGLOBAL_TAG	'5'
#define ABSGLOBAL_TAG	'6'
#define CKSUM_TAG	'7'
#define NOCKSUM_TAG	'8'
#define ABSORG_TAG	'9'
#define RELORG_TAG	'A'
#define ABSDATA_TAG	'B'
#define RELDATA_TAG	'C'
#define EXTVALUE_TAG	'D'
#define RELBYTE_TAG	'E'
#define EOR_TAG		'F'

#define CHARSPERREC	66  /* Chars per object record */
#define CHARSPERCASREC	73  /* Chars per cassette object record */
#define WORDTAGLEN	5   /* Word + Object Tag length */
#define WORDLEN		4   /* Word length */
#define EXTRNLEN	11  /* Tag + SYMBOL + addr */
#define GLOBALLEN	11  /* Tag + SYMBOL + addr */
#define IDTLEN		11  /* Tag + IDT + length */
#define IDTCOLUMN	72  /* Where to put the IDT */
#define SEQUENCENUM	76  /* Where to put the sequence */

#define TIMEOFFSET 17
#define DATEOFFSET 27
#define CREATOROFFSET 38

extern int decodeinst (uint16);
extern int disloader (FILE *, int, char *);
extern void disprog (int);
extern SymNode *symlookup (char *, char *, int);
extern SymNode *vallookup (int, int, int);
extern SymNode *extlookup (int, int);
extern SymNode *symadd (char *, int);
