/***********************************************************************
*
* MKDISK - Make a TI980 disk image file.
*
* Changes:
*      12/17/03   DGP   Original
*      07/15/04   DGP   Corrected DS80 info.
*      07/21/11   DGP   Hacked from TI990 version.
*
***********************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>

#define NORMAL		0
#define ABORT		16

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#include "simdsk.h"

/***********************************************************************
* dskwriteint - Write an integer.
***********************************************************************/

static int
dskwriteint (FILE *fd, int num)
{

   if (fputc ((num >> 24) & 0xFF, fd) < 0) return (-1);
   if (fputc ((num >> 16) & 0xFF, fd) < 0) return (-1);
   if (fputc ((num >> 8) & 0xFF, fd) < 0) return (-1);
   if (fputc (num & 0xFF, fd) < 0) return (-1);
   return (0);
}

/***********************************************************************
* main
***********************************************************************/

int
main (int argc, char **argv)
{
   FILE *dfd;
   char *bp;
   char *diskfile;
   char *diskname;
   int i;
   int diskndx;
   int geoonly;
   int disksize;
   int verbose;

   verbose = FALSE;
   geoonly = FALSE;
   diskfile = NULL;
   diskname = NULL;

   /*
   ** Scan off args 
   */

   for (i = 1; i < argc; i++)
   {
      bp = argv[i];

      if (*bp == '-')
      {
         for (bp++; *bp; bp++) switch (*bp)
         {
	 case 'd':
	    i++;
	    diskname = argv[i];
	    break;

	 case 'g':
	    geoonly = TRUE;
	    break;

	 case 'v':
	    verbose = TRUE;
	    break;

         default:
      USAGE:
	    printf ("usage: mkdisk [-options] disk.file\n");
            printf (" options:\n");
	    printf ("    -d dskmodel  - Disk model\n");
	    printf ("          ");
	    for (i = 0; i < MAXDISKS; i++)
	       printf ("%s ", disks[i].model);
	    printf ("\n");
	    printf ("    -g           - Disk geometry\n");
            printf ("    -v           - Verbose output\n");
	    return (ABORT);
         }
      }

      else
      {
         if (diskfile) goto USAGE;
         diskfile = argv[i];
      }

   }

   if (!diskfile) goto USAGE;
   if (!diskname) diskname = "DS31";

   /*
   ** Check disk model
   */

   for (diskndx = 0; diskndx < MAXDISKS; diskndx++)
   {
      if (!strcmp (diskname, disks[diskndx].model)) break;
   }
   if (diskndx == MAXDISKS)
   {
      printf ("mkdisk: Unknown disk model: %s\n", diskname);
      exit (ABORT);
   }

   /*
   ** Calulate the size of the disk
   */

   disksize = (disks[diskndx].bytsec + disks[diskndx].overhead) *
	disks[diskndx].sectrk * disks[diskndx].heads * disks[diskndx].cyls;

   if (verbose)
   {
      float size;
      size = disksize / 1048576.0;
      printf ("mkdisk: create disk %s on file %s\n", diskname, diskfile);
      printf (
"Geometry: cyls = %d, heads = %d, sec/trk = %d, byte/sec = %d, overhead = %d\n",
		  disks[diskndx].cyls, disks[diskndx].heads,
		  disks[diskndx].sectrk, disks[diskndx].bytsec,
		  disks[diskndx].overhead);
      printf (" Disk size = %6.2f MB\n", size);
   }

   /*
   ** Open the disk image file
   */

   if ((dfd = fopen (diskfile, "wb")) == NULL)
   {
      perror ("mkdisk: Can't open diskfile");
      exit (ABORT);
   }

   /*
   ** Write the disk geometry information
   */

   dskwriteint (dfd, disks[diskndx].cyls);
   dskwriteint (dfd, (disks[diskndx].type << 28) | (diskndx << 16) |
		      disks[diskndx].heads);
   dskwriteint (dfd, (disks[diskndx].overhead << 16) | disks[diskndx].sectrk);
   dskwriteint (dfd, disks[diskndx].bytsec);

   /*
   ** Write the disk image
   */

   if (!geoonly) for (i = 0; i < disksize; i++)
   {
      fputc ('\0', dfd);
   }
   
   fclose (dfd);
   return (NORMAL);

}
