/***********************************************************************
*
* MKTAPE - Make a tape image file.
*
* Changes:
*      03/31/05   DGP   Original
*
***********************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>

#define NORMAL		0
#define ABORT		16

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/***********************************************************************
* tapewriteint - Write an integer.
***********************************************************************/

static void
tapewriteint (FILE *fd, int v)
{
   v &= 0x7FFF;
   fputc (v & 0xFF, fd);
   fputc ((v >> 8) & 0xFF, fd);
   fputc ((v >> 16) & 0xFF, fd);
   fputc ((v >> 24) & 0xFF, fd);
}

/***********************************************************************
* main
***********************************************************************/

int
main (int argc, char **argv)
{
   FILE *ifd;
   FILE *tfd;
   char *bp;
   char *tapefile;
   char *infile;
   int i;
   int done;
   int verbose;
   int reclen;
   int binary;
   char inbuf[32768];

   verbose = FALSE;
   binary = FALSE;
   tapefile = NULL;
   infile = NULL;
   reclen = 80;

   /*
   ** Scan off args 
   */

   for (i = 1; i < argc; i++)
   {
      bp = argv[i];

      if (*bp == '-')
      {
         for (bp++; *bp; bp++) switch (*bp)
         {
	 case 'b':
	    binary = TRUE;
	    break;

	 case 'r':
	    i++;
	    reclen = atoi (argv[i]);
	    break;

	 case 'v':
	    verbose = TRUE;
	    break;

         default:
      USAGE:
	    printf ("usage: mktape [-options] inputfile tape.file\n");
            printf (" options:\n");
	    printf ("    -b           - Binary mode\n");
            printf ("    -r NN        - Record length\n");
            printf ("    -v           - Verbose output\n");
	    return (ABORT);
         }
      }
      else if (!infile)
      {
         infile = argv[i];
      }
      else if (!tapefile)
      {
        tapefile = argv[i];
      }
      else
      {
         goto USAGE;
      }

   }

   if (!tapefile && !infile) goto USAGE;

   /*
   ** Open the input file
   */

   if (verbose) printf ("mktape: Opening %s for input\n", infile);
   if (binary)
      ifd = fopen (infile, "rb");
   else
      ifd = fopen (infile, "r");
   if (ifd == NULL)
   {
      perror ("mktape: Can't open input");
      exit (ABORT);
   }

   /*
   ** Open the tape image file
   */

   if (verbose) printf ("mktape: Opening %s for output\n", tapefile);
   if ((tfd = fopen (tapefile, "wb")) == NULL)
   {
      perror ("mktape: Can't open tapefile");
      fclose (ifd);
      exit (ABORT);
   }

   done = FALSE;
   while (!done)
   {
      if (binary)
      {
	 size_t inlen;

	 inlen = fread (inbuf, 1, reclen, ifd);
	 if (inlen != reclen)
	 {
	    done = TRUE;
	 }
	 if (verbose) printf ("mktape: Binary read %d bytes, reclen %d\n",
	       inlen, reclen);
      }
      else
      {
	 if (fgets (inbuf, sizeof(inbuf), ifd) == NULL)
	 {
	    done = TRUE;
	 }
	 else
	 {
	    i = strlen(inbuf) - 1;
	    if (verbose) printf ("mktape: ASCII read %d bytes\n", i);
	    if (i >= reclen) inbuf[reclen] = '\0';
	    else
	    {
	       for (; i < reclen; i++)
	       {
		  inbuf[i] = ' ';
	       }
	    }
	 }
      }

      if (!done)
      {
	 tapewriteint (tfd, reclen);
	 fwrite (inbuf, reclen, 1, tfd);
	 tapewriteint (tfd, reclen);
      }
   }
   tapewriteint (tfd, 0);

   fclose (ifd);
   fclose (tfd);
   return (NORMAL);
}
