/***********************************************************************
*
* RDTAPE - Read a tape image file.
*
* Changes:
*      03/31/05   DGP   Original
*
***********************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>

#define NORMAL		0
#define ABORT		16

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#include "hexdump.h"

/***********************************************************************
* tapereadint - Read an integer.
***********************************************************************/

static int
tapereadint (FILE *fd)
{
   int r;

   r = fgetc (fd);
   r = r | (fgetc (fd) << 8);
   r = r | (fgetc (fd) << 16);
   r = r | (fgetc (fd) << 24);
   if (feof(fd)) return (-2);
   return (r & 0x7FFF);
}

/***********************************************************************
* main
***********************************************************************/

int
main (int argc, char **argv)
{
   FILE *tfd;
   FILE *ofd;
   char *bp;
   char *tapefile;
   char *outfile;
   int i;
   int done;
   int verbose;
   int binary;
   int strip;
   int toss;
   int reccnt;
   int reclen;
   char inbuf[1024];

   ofd = stdout;

   verbose = FALSE;
   binary = FALSE;
   strip = FALSE;
   toss = FALSE;
   tapefile = NULL;
   outfile = NULL;
   reclen = 80;

   /*
   ** Scan off args 
   */

   for (i = 1; i < argc; i++)
   {
      bp = argv[i];

      if (*bp == '-')
      {
         for (bp++; *bp; bp++) switch (*bp)
         {
	 case 'b':
	    binary = TRUE;
	    break;

	 case 'r':
	    i++;
	    reclen = atoi (argv[i]);
	    break;

	 case 's':
	    strip = TRUE;
	    break;

	 case 't':
	    toss = TRUE;
	    break;

	 case 'v':
	    verbose = TRUE;
	    break;

         default:
      USAGE:
	    printf ("usage: rdtape [-options] tape.file [out.file]\n");
            printf (" options:\n");
            printf ("    -b           - Binary output\n");
            printf ("    -r NN        - Record length\n");
            printf ("    -s           - Strip leading CRLF\n");
            printf ("    -t           - Toss first record CC header\n");
            printf ("    -v           - Verbose output\n");
	    return (ABORT);
         }
      }
      else if (!tapefile)
      {
        tapefile = argv[i];
      }
      else if (!outfile)
      {
        outfile = argv[i];
      }
      else
      {
         goto USAGE;
      }

   }

   if (!tapefile) goto USAGE;

   /*
   ** Open the tape image file
   */

   if ((tfd = fopen (tapefile, "rb")) == NULL)
   {
      perror ("rdtape: Can't open tapefile");
      exit (ABORT);
   }

   if (outfile)
   {
      if (binary) strcpy (inbuf, "wb");
      else        strcpy (inbuf, "w");

      if ((ofd = fopen (outfile, inbuf)) == NULL)
      {
	 perror ("rdtape: Can't open outfile");
	 exit (ABORT);
      }
   }

   if (verbose)
   {
      fprintf (stderr, "rdtape: Reading file %s\n", tapefile);
      fprintf (stderr, "        Writing file %s\n",
				 tapefile ? tapefile : "stdout");
      fprintf (stderr, "        Reclen = %d\n", reclen);
      fprintf (stderr, "        Binary = %s\n", binary ? "TRUE" : "FALSE");
      fprintf (stderr, "        Strip  = %s\n", strip ? "TRUE" : "FALSE");
      fprintf (stderr, "        Toss   = %s\n", toss ? "TRUE" : "FALSE");
   }

   reccnt = 0;
   done = FALSE;
   while (!done)
   {
      i = tapereadint (tfd);
      if (i <= 0) break;
      if (verbose && !toss)
      {
         fprintf (stderr, "   read record %05d, length = %05d\n", ++reccnt, i);
	 HEXDUMP (stderr, inbuf, i, 0);
      }
      fread (inbuf, i, 1, tfd);
      tapereadint (tfd);
      if (i >= reclen) inbuf[reclen] = '\0';
      if (!toss)
      {
	 if (binary)
	 {
	    inbuf[i++] = '\n';
	    fwrite (inbuf, i, 1, ofd);
	 }
	 else
	 {
	    if (strip)
	    {
	       if (inbuf[0] == '\r' && inbuf[1] == '\n')
		  memmove (inbuf, &inbuf[2], i);
	    }
	    strcat (inbuf, "\n");
	    fputs (inbuf, ofd);
	 }
      }
      else
         toss = FALSE;
   }

   fclose (tfd);
   if (ofd != stdout) fclose (ofd);
   return (NORMAL);
}
